/*
 * CipherOutputStream.java
 *
 * This class is part of our JCE 1.2 cleanroom implementation
 *
 * Copyright (c) 2000 Virtual Unlimited B.V.
 *
 * Author: Bob Deblier <bob@virtualunlimited.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package javax.crypto;

import java.io.*;

public class CipherOutputStream extends FilterOutputStream
{
	private Cipher cipher;
	private OutputStream filter;

	private byte[] filterBuffer;
	private byte[] updateBuffer;
	
	protected CipherOutputStream(OutputStream os)
	{
		this(os, new NullCipher());
	}

	public CipherOutputStream(OutputStream os, Cipher c)
	{
		super(os);
		cipher = c;
		filter = os;
		filterBuffer = new byte[1];
		updateBuffer = null;
	}

	public void close() throws IOException
	{
		try
		{
			updateBuffer = cipher.doFinal();
		}
		catch (IllegalBlockSizeException e)
		{
			updateBuffer = null;
		}
		catch (BadPaddingException e)
		{
			updateBuffer = null;
		}
		try
		{
			flush();
		}
		catch (IOException e)
		{
		}
		filter.close();
	}

	public void flush() throws IOException
	{
		if (updateBuffer != null)
		{
			filter.write(updateBuffer);
			updateBuffer = null;
		}
		filter.flush();
	}

	public void write(int b) throws IOException
	{
		filterBuffer[0] = (byte) b;
		updateBuffer = cipher.update(filterBuffer, 0, 1);
		if (updateBuffer != null)
		{
			filter.write(updateBuffer);
			updateBuffer = null;
		}
	}

	public void write(byte[] b) throws IOException
	{
		write(b, 0, b.length);
	}

	public void write(byte[] b, int off, int len) throws IOException
	{
		updateBuffer = cipher.update(b, off, len);
		if (updateBuffer != null)
		{
			filter.write(updateBuffer);
			updateBuffer = null;
		}
	}
}
